<!DOCTYPE html>
<html lang="en">
<head>
    <title>{{config.API_TITLE}} v{{config.API_VERSION}}</title>
    <meta charset="utf-8"/>
    <meta content="width=device-width, initial-scale=1" name="viewport">
    {{custom_headers|safe}}
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/swagger-ui/5.17.14/swagger-ui.css" />
</head>
<body>
<div id="swagger-ui"></div>
<script src="https://cdnjs.cloudflare.com/ajax/libs/swagger-ui/5.17.14/swagger-ui-bundle.js"></script>
<script>
    SwaggerUIBundle({
        url: "{{ url_for('specification.get_swagger_spec') }}",
        dom_id: '#swagger-ui'
    });
</script>
</body>
{{custom_footers|safe}}
</html>
