# Fetch all items
query {
  all_items {
    id
    name
  }
}

# Create a new item
mutation {
  create_item(name: "Biscuit") {
    id
    name
  }
}

# Filter and paginate
query {
  all_items(name: "Biscuit", limit: 5, offset: 0) {
    id
    name
  }
}

# Update and delete
mutation {
  update_item(id: 1, name: "Cookie") { id name }
  delete_item(id: 1)
}

# Query with relationships
query {
  all_items {
    id
    name
    category { id name }
  }
}
